@@ -19,99 +19,104 @@
 #include "glib.h"
 
 
-typedef struct _GRealListAllocator GRealListAllocator;
-
-struct _GRealListAllocator
+struct _GAllocator /* from gmem.c */
 {
-  GMemChunk *list_mem_chunk;
-  GList	    *free_list;
+  gchar         *name;
+  guint16        n_preallocs;
+  guint          is_unused : 1;
+  guint          type : 4;
+  GAllocator    *last;
+  GMemChunk     *mem_chunk;
+  GList		*free_lists; /* implementation specific */
 };
 
+static GAllocator	*current_allocator = NULL;
 
-static GRealListAllocator *default_allocator = NULL;
-static GRealListAllocator *current_allocator = NULL;
-
-
-GListAllocator*
-g_list_allocator_new (void)
+void
+g_list_push_allocator (GAllocator *allocator)
 {
-  GRealListAllocator* allocator = g_new (GRealListAllocator, 1);
-  
-  allocator->list_mem_chunk = NULL;
-  allocator->free_list = NULL;
-  
-  return (GListAllocator*) allocator;
+  g_return_if_fail (allocator != NULL);
+  g_return_if_fail (allocator->is_unused == TRUE);
+
+  if (allocator->type != G_ALLOCATOR_LIST)
+    {
+      allocator->type = G_ALLOCATOR_LIST;
+      if (allocator->mem_chunk)
+	{
+	  g_mem_chunk_destroy (allocator->mem_chunk);
+	  allocator->mem_chunk = NULL;
+	}
+    }
+
+  if (!allocator->mem_chunk)
+    {
+      allocator->mem_chunk = g_mem_chunk_new (allocator->name,
+					      sizeof (GList),
+					      sizeof (GList) * allocator->n_preallocs,
+					      G_ALLOC_ONLY);
+      allocator->free_lists = NULL;
+    }
+
+  allocator->is_unused = FALSE;
+  allocator->last = current_allocator;
+  current_allocator = allocator;
 }
 
 void
-g_list_allocator_free (GListAllocator* fallocator)
+g_list_pop_allocator (void)
 {
-  GRealListAllocator* allocator = (GRealListAllocator *) fallocator;
-  
-  if (allocator && allocator->list_mem_chunk)
-    g_mem_chunk_destroy (allocator->list_mem_chunk);
-  if (allocator)
-    g_free (allocator);
-}
-
-GListAllocator*
-g_list_set_allocator (GListAllocator* fallocator)
-{
-  GRealListAllocator* allocator = (GRealListAllocator *) fallocator;
-  GRealListAllocator* old_allocator = current_allocator;
-  
-  if (allocator)
-    current_allocator = allocator;
-  else
+  if (current_allocator)
     {
-      if (!default_allocator)
-	default_allocator = (GRealListAllocator*) g_list_allocator_new ();
-      current_allocator = default_allocator;
-    }
-  
-  if (!current_allocator->list_mem_chunk)
-    current_allocator->list_mem_chunk = g_mem_chunk_new ("list mem chunk",
-							 sizeof (GList),
-							 1024,
-							 G_ALLOC_ONLY);
-  
-  return (GListAllocator*) (old_allocator == default_allocator ? NULL : old_allocator);
-}
+      GAllocator *allocator;
 
+      allocator = current_allocator;
+      current_allocator = allocator->last;
+      allocator->last = NULL;
+      allocator->is_unused = TRUE;
+    }
+}
 
 GList*
 g_list_alloc (void)
 {
-  GList *new_list;
-  
-  g_list_set_allocator (NULL);
-  if (current_allocator->free_list)
+  GList *list;
+
+  if (!current_allocator)
+    g_list_push_allocator (g_allocator_new ("GLib default GList allocator", 1024));
+
+  if (!current_allocator->free_lists)
     {
-      new_list = current_allocator->free_list;
-      current_allocator->free_list = current_allocator->free_list->next;
+      list = g_chunk_new (GList, current_allocator->mem_chunk);
+      list->data = NULL;
     }
   else
     {
-      new_list = g_chunk_new (GList, current_allocator->list_mem_chunk);
+      if (current_allocator->free_lists->data)
+	{
+	  list = current_allocator->free_lists->data;
+	  current_allocator->free_lists->data = list->next;
+	  list->data = NULL;
+	}
+      else
+	{
+	  list = current_allocator->free_lists;
+	  current_allocator->free_lists = list->next;
+	}
     }
+  list->next = NULL;
+  list->prev = NULL;
   
-  new_list->data = NULL;
-  new_list->next = NULL;
-  new_list->prev = NULL;
-  
-  return new_list;
+  return list;
 }
 
 void
 g_list_free (GList *list)
 {
-  GList *last;
-
   if (list)
     {
-      last = g_list_last (list);
-      last->next = current_allocator->free_list;
-      current_allocator->free_list = list;
+      list->data = list->next;
+      list->next = current_allocator->free_lists;
+      current_allocator->free_lists = list;
     }
 }
 
@@ -120,8 +125,9 @@ g_list_free_1 (GList *list)
 {
   if (list)
     {
-      list->next = current_allocator->free_list;
-      current_allocator->free_list = list;
+      list->data = NULL;
+      list->next = current_allocator->free_lists;
+      current_allocator->free_lists = list;
     }
 }
 
@@ -273,6 +279,32 @@ g_list_remove_link (GList *list,
   return list;
 }
 
+GList*
+g_list_copy (GList *list)
+{
+  GList *new_list = NULL;
+
+  if (list)
+    {
+      GList *last;
+
+      new_list = g_list_alloc ();
+      new_list->data = list->data;
+      last = new_list;
+      list = list->next;
+      while (list)
+	{
+	  last->next = g_list_alloc ();
+	  last->next->prev = last;
+	  last = last->next;
+	  last->data = list->data;
+	  list = list->next;
+	}
+    }
+
+  return new_list;
+}
+
 GList*
 g_list_reverse (GList *list)
 {
@@ -542,3 +574,62 @@ g_list_sort (GList       *list,
 			    g_list_sort (l2,   compare_func),
 			    compare_func);
 }
+
+GList* 
+g_list_sort2 (GList       *list,
+	      GCompareFunc compare_func)
+{
+  GSList *runs = NULL;
+  GList *tmp;
+
+  /* Degenerate case.  */
+  if (!list) return NULL;
+
+  /* Assume: list = [12,2,4,11,2,4,6,1,1,12].  */
+  for (tmp = list; tmp; )
+    {
+      GList *tmp2;
+      for (tmp2 = tmp;
+	   tmp2->next && compare_func (tmp2->data, tmp2->next->data) <= 0;
+	   tmp2 = tmp2->next)
+	/* Nothing */;
+      runs = g_slist_append (runs, tmp);
+      tmp = tmp2->next;
+      tmp2->next = NULL;
+    }
+  /* Now: runs = [[12],[2,4,11],[2,4,6],[1,1,12]].  */
+  
+  while (runs->next)
+    {
+      /* We have more than one run.  Merge pairwise.  */
+      GSList *dst, *src, *dstprev = NULL;
+      dst = src = runs;
+      while (src && src->next)
+	{
+	  dst->data = g_list_sort_merge (src->data,
+					 src->next->data,
+					 compare_func);
+	  dstprev = dst;
+	  dst = dst->next;
+	  src = src->next->next;
+	}
+
+      /* If number of runs was odd, just keep the last.  */
+      if (src)
+	{
+	  dst->data = src->data;
+	  dstprev = dst;
+	  dst = dst->next;
+	}
+
+      dstprev->next = NULL;
+      g_slist_free (dst);
+    }
+
+  /* After 1st loop: runs = [[2,4,11,12],[1,1,2,4,6,12]].  */
+  /* After 2nd loop: runs = [[1,1,2,2,4,4,6,11,12,12]].  */
+
+  list = runs->data;
+  g_slist_free (runs);
+  return list;
+}
